/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <POURRIER Thibaud> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * Poul-Henning Kamp.
 * ----------------------------------------------------------------------------
 */

/******************************************************************************
		"main.c"
		This file is the main file of the project INSOLED
		CPU  = ATtiny2313
		FCPU = 4MHz.
*******************************************************************************/


#include "main.h"

//****************************************************** Main
int main()
{
	Init();
//	uchar i = 0;
#ifdef DEBUG
	Hello_led();
#endif

	do
	{
		Enter_target();
		Uncount();		// We count from n-1 to 0 (for five second, we count from 4 to 0)
		CMD_ON;			// Switch on LED panels!

		// Display infos
		Write_str("Il reste   \"  \'", 0x00);
	
		TIM0_ON;	// Start timer0 (and uncounter)
		do
		{
			Write_nbr(Current[0], TARGET_ADD, NOTHING);
			Write_nbr(Current[1], TARGET_ADD+1, NOTHING);
			Write_nbr(Current[2], TARGET_ADD+3, NOTHING);
			Write_nbr(Current[3], TARGET_ADD+4, NOTHING);
			dms(100);
		}while(Compare() && OK);	// While Current != 0 XOR OK is pressed
	
		TIM0_OFF;
		CMD_OFF;	// Switch off LED panels!
	
		Write_nbr(Current[3], TARGET_ADD+4, NOTHING);	// Refresh last digit for display 0

		Buzz();

		Write_str("Restart?", 0x00);
	
		while(OK);
	} while(42);	// Vers l'infini et au del!!!
}


//****************************************************** Init
void Init()
{
	uchar i = 0;

	cli();	//interrupts disable

	Init_screen();
	Write_str("Init...", 0x00);

// I/O
#ifdef DEBUG
	 DDRD |= (1<<PD3);		// PD3 out (LED)
#endif
	PORTD |= 0b0111;		// PD0..2 in (-/>/+), PullUp enabled
	PORTA |= 0b0011;		// PA0..1 in (</OK), PullUp enabled
	 DDRD |= (1<<PD6);		// PD6 out (command)
	 DDRD |= (1<<PD6);		// PD5 out (Buzzer)

// Uncounter (on timer0)
	TCCR0A |= (1<<WGM01)|(0<<WGM00);
	TCCR0B |= (0<<WGM02) | (0<<CS02)|(1<<CS01)|(1<<CS00);	// CTC mode selected | Div = 64
	OCR0A   = 249;											// From 0 to 249 = 250. See interrupt for more details
//	TIMSK  |= (1<<OCIE0A);

// Vars
	for(i=0 ; i<DIGIT ; i++)
	{
		Current[i] = 0;
		if(eeprom_read_byte((const uchar*)(uint16_t)i) == 255)
			eeprom_write_byte((uchar*)(uint16_t)i, (uchar)0);
		Target[i] = eeprom_read_byte((const uchar*)(uint16_t)i);	// Read eeprom to set Target
	}
	Count = 0;

	sei();	// Enable interrupts = SREG |= _BV(7)
}


//****************************************************** Enter target
void Enter_target()
{
	uchar i = 0, pos = DIGIT;	// pos: cursor position

// Display
	CURSOR_ON;
	Clear_screen();
	Write_str("Temps?     \"  \'", 0x00);

	Write_nbr(Target[0], TARGET_ADD, NOTHING);
	Write_nbr(Target[1], TARGET_ADD+1, NOTHING);
	Write_nbr(Target[2], TARGET_ADD+3, NOTHING);
	Write_nbr(Target[3], TARGET_ADD+4, NOTHING);

	Write_cmd(128+TARGET_ADD+pos);	// Cursor display
	
	dms(250);	// Switch debouncer
	while(OK)	// While OK button is not pressed
	{
		if(!PLUS)		// + pressed
		{
			if(pos >= 3)	// pos = 3 or 4
			{
				if(Target[pos-1] >= 9)
					Target[pos-1] = 0;
				else
					Target[pos-1]++;

				Write_nbr(Target[pos-1], TARGET_ADD+pos, 42);
			}
			else			// pos = 1 or 2
			{
				if(Target[pos] >= 9)
					Target[pos] = 0;
				else
					Target[pos]++;

				Write_nbr(Target[pos], TARGET_ADD+pos, 42);
			}
		}
		else if(!MINUS)	// - pressed
		{
			if(pos >= 3)	// ":" gap
			{
				if(Target[pos-1] <= 0)
					Target[pos-1] = 9;
				else
					Target[pos-1]--;

				Write_nbr(Target[pos-1], TARGET_ADD+pos, 42);
			}
			else			// pos = 1 or 2
			{
				if(Target[pos] <= 0)
					Target[pos] = 9;
				else
					Target[pos]--;

				Write_nbr(Target[pos], TARGET_ADD+pos, 42);
			}
		}
		else if(!RIGHT)	// > pressed
		{
			if(pos == (DIGIT))
				pos = 0;
			else if(pos == 1)	// ":" gap
				pos+=2;
			else
				pos++;
		}
		else if(!LEFT)	// < pressed
		{
			if(pos == 0)
				pos = (DIGIT);
			else if(pos == 3)	// ":" gap
				pos-=2;
			else
				pos--;
		}
						
		Write_cmd(128+TARGET_ADD+pos);	// Replace cursor
		dms(150);						// Switch debouncer
	}
	
	for(i=0 ; i<DIGIT ; i++)
	{
		eeprom_write_byte((uchar*)(uint16_t)i, (uchar)Target[i]);	// Save Target
		Current[i] = Target[i];										// Put Target in Current
	}

	CURSOR_OFF;	// Cursor display
}


//****************************************************** Counter interrupt (on timer0)
/*
	Divider = 64 -> One timer cycle = 16s -> 250*16s = 4ms -> 250*4ms = 1s.
	OCR0A = 250
*/
ISR(TIMER0_COMPA_vect)
{
	if(Count >=249)	// From 0 to 249 = 250
	{
		Uncount();
		Count = 0;
	}
	else
		Count++;
}


//****************************************************** Uncounter
void Uncount()
{
	uchar i = 0;

	Current[3]--;
	
	for(i=3 ; i>=1 ; i--)
		if(Current[i] == 255)	// = -1
		{
			if(i == 2)
				Current[i] = 5;
			else
				Current[i] = 9;
			Current[i-1]--;
		}		
}


//****************************************************** Compare
uchar Compare()
{
	uchar i = 0;
	
	for(i=0 ; i<DIGIT ; i++)
		if(Current[i] != 0)
			return 42;	// Return 1 if Current != 0
	
	return 0;			// Return 0 if Current = 0
}


//****************************************************** Buzzer
void Buzz()
{
	uchar i = 0, y = 0;
	
	for(y=0 ; y<5 ; y++)
	{
		for(i=0 ; i<100; i++)
		{
			BUZZER_ON;
			dms(1);
			BUZZER_OFF;
			dms(1);
		}
		dms(250);
	}
}


//****************************************************** DEBUG
#ifdef DEBUG

void Hello_led()
{
	uchar i = 0;
	
	for(i=0 ; i<=3; i++)
	{
		LED_ON;
		dms(500);
		LED_OFF;
		dms(500);
	}
}

#endif

