/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <POURRIER Thibaud> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * Poul-Henning Kamp.
 * ----------------------------------------------------------------------------
 */

/******************************************************************************
		"main.h"
		This file is the header of the project INSOLED.
*******************************************************************************/


#ifndef mainh
#define mainh

// Libraries
#include "avr/io.h"
#include "avr/interrupt.h"
#include "avr/eeprom.h"

#define F_CPU	4000000UL	// 4MHz
#include "util/delay.h"

// GCC Macros
#define dms(m)	_delay_ms(m)
#define dus(u)	_delay_us(u)
typedef unsigned char	uchar;
typedef unsigned int	uint;


//------------------ Prototypes ------------------
int  main();
void Init();
void Enter_target();
void Uncount();
uchar Compare();
void Buzz();

//------------------ Macros ----------------------
// Buttons
#define PLUS		(PIND & _BV(PD2))
#define MINUS		(PIND & _BV(PD0))
#define RIGHT		(PIND & _BV(PD1))
#define LEFT		(PINA & _BV(PA1))
#define OK			(PINA & _BV(PA0))
// Command
#define CMD_ON		(PORTB |= _BV(PB4))
#define CMD_OFF		(PORTB &= ~_BV(PB4))
// Buzzer
#define BUZZER_ON	(PORTD |= _BV(PD6))
#define BUZZER_OFF	(PORTD &= ~_BV(PD6))
// Timer0
#define TIM0_ON		(TIMSK |= _BV(OCIE0A))
#define TIM0_OFF	(TIMSK &= ~_BV(OCIE0A))

//------------------ Constants -------------------
#define DIGIT		4		// Number of digits (2 to 8)
#define TARGET_ADD	0x41	// Screen add of the target number

//------------------ Vars ------------------------
volatile uchar Target[DIGIT];	// Target number of spins, set by user
volatile uchar Current[DIGIT];	// Current number of spins, count by hardware
volatile uchar Count;			// Used in timer0 interrupt

//------------------ Flags -----------------------
//volatile unsigned char FLG_Global;
// Bit		|	7	|	6	|	5	|	4	||	3	|	2	|	1	|	0	|
// Name		|  		|		|  		| 		|| 		| 		| 		| 		|
// Default	|	1	|	0	|	0	|	X	||	X	|	X	|	X	|	X	|
/*#define _FLG		(REG_Global_Flg & _BV(7))
#define _FLG_ON	(REG_Global_Flg |= _BV(7))
#define _FLG_OFF	(REG_Global_Flg &= ~_BV(7))*/



/************************************************************
							DEBUG
************************************************************/
//#define DEBUG		// Uncomment this line for debuging mode
#ifdef DEBUG
#define LED_ON			(PORTD |= _BV(PD3))
#define LED_OFF			(PORTD &= ~_BV(PD3))
void Hello_led();
#endif
/************************************************************/




// Headers
#include "screen4b.h"

#endif	//#define mainh
